@@ -1,5 +1,5 @@
 package Bio::ToolBox::Data;
-our $VERSION = '1.68';
+our $VERSION = '1.69';
 
 =head1 NAME
 
@@ -483,6 +483,10 @@ in ascending (left to right) order.
 
 Returns the number of columns in the Data table. 
 
+=item number_rows
+
+Returns the number of rows in the Data table.
+
 =item last_column
 
 Returns the array index number for the last (right most) 
@@ -897,15 +901,20 @@ Pass the name of the filename.
 
 =item taste_file
 
-  my $flavor = $Data->taste_file($filename);
-  # returns gff, bed, ucsc, or undef
+  my ($flavor, $format) = $Data->taste_file($filename);
+  # $flavor is generic term: gff, bed, ucsc, or undef
+  # $format is specific term: gtf, gff3, narrowPeak, genePred, etc.
+
+Tastes, or checks, a file for a certain flavor, or known gene file formats.
+Useful for determining if the file represents a known gene table format
+that lacks a defined file extension, e.g. UCSC formats. This can be based
+on the file extension, metadata headers, and/or file contents from the
+first 10 lines. Returns two strings: the first is a generic flavor, and the
+second is a more specific format, if applicable. Generic flavor values will
+be one of `gff`, `bed`, `ucsc`, or `undefined`. These correlate to specific
+Parser adapters. Specific formats could be any number of possibilities, for
+example `undefined`, `gtf`, `gff3`, `narrowPeak`, `genePred`, etc.  
 
-Tastes, or checks, a file for a certain flavor, or known gene file formats. 
-This is based on file extension, metadata headers, and/or file content 
-in the first 10 lines or so. Returns a string based on the file format.
-Values include gff, bed, ucsc, or undefined. Useful for determining if 
-the file represents a known gene table format that lacks a defined file 
-extension, e.g. UCSC formats.
 
 =item save
 
@@ -1091,7 +1100,7 @@ use Bio::ToolBox::db_helper qw(
 	get_new_genome_list
 	get_db_feature
 );
-use Bio::ToolBox::utility qw(simplify_dataset_name);
+use Bio::ToolBox::utility qw(simplify_dataset_name sane_chromo_sort);
 use Module::Load;
 
 1;
@@ -1102,6 +1111,9 @@ use Module::Load;
 sub new {
 	my $class = shift;
 	my %args  = @_;
+	if (ref($class)) {
+		$class = ref($class);
+	}
 	
 	# check for important arguments
 	$args{features} ||= $args{feature} || 'gene';
@@ -1150,6 +1162,26 @@ sub new {
 		$args{data} = $self;
 		my $result;
 		if ($args{features} eq 'genome') {
+			# create a genomic interval list
+			# first must parse any exclusion list provided as a new Data object
+			$args{blacklist} ||= $args{exclude} || undef;
+			if (defined $args{blacklist}) {
+				my $exclusion_Data = $self->new(
+					file => $args{blacklist},
+					parse => 0 # we don't need to parse
+				);
+				if ($exclusion_Data and $exclusion_Data->feature_type eq 'coordinate') {
+					printf "   Loaded %d exclusion list items\n", 
+						$exclusion_Data->number_rows;
+					delete $args{blacklist};
+					delete $args{exclude};
+					$args{exclude} = $exclusion_Data;
+				}
+				else {
+					carp " Cannot not load exclusion coordinate list file $args{blacklist}!";
+					return;
+				}
+			}
 			$result = get_new_genome_list(%args);
 		}
 		else {
@@ -1259,13 +1291,14 @@ sub parse_table {
 	}
 	
 	# the file format determines the parser class
-	my $flavor = $self->taste_file($file) or return;
+	my ($flavor, $format) = $self->taste_file($file) or return;
 	my $class = 'Bio::ToolBox::parser::' . $flavor;
 	eval {load $class};
 	if ($@) {
 		carp "unable to load $class! cannot parse $flavor!";
 		return;
 	}
+	$self->format($format); # specify the specific format
 	
 	# open parser
 	my $parser = $class->new() or return;
@@ -1869,72 +1902,46 @@ sub gsort_data {
 	}
 	my $chromo_i = $self->chromo_column;
 	my $start_i  = $self->start_column;
+	my $stop_i   = $self->stop_column || $start_i;
 	
-	# Load the data into a temporary hash
-	# The datalines will be put into a hash of hashes: The first key will be 
-	# the chromosome name, the second hash will be the start value.
-	# 
-	# To deal with some chromosomes that don't have numbers (e.g. chrM), we'll
-	# use two separate hashes: one is for numbers, the other for strings
-	# when it comes time to sort, we'll put the numbers first, then strings
-	
-	my %num_datahash;
-	my %str_datahash;
-	for my $row (1 .. $self->last_row) { 
-		
-		my $startvalue = $self->value($row, $start_i);
-		
-		# put the dataline into the appropriate temporary hash
-		if ($self->value($row, $chromo_i) =~ /^(?:chr)?(\d+)$/) {
-			# dealing with a numeric chromosome name
-			# restricting to either chr2 or just 2 but not 2-micron
-			my $chromovalue = $1;
-			while (exists $num_datahash{$chromovalue}{$startvalue}) { 
-				# if another item already exists at this location
-				# add a really small number to bump it up and make it unique
-				$startvalue += 0.001; 
-			}
-			$num_datahash{$chromovalue}{$startvalue} = $self->{data_table}->[$row];
-		} 
-		else {
-			# dealing with a non-numeric chromosome name
-			my $chromovalue = $self->value($row, $chromo_i);
-			# use the entire chromosome name as key
-			while (exists $str_datahash{$chromovalue}{$startvalue}) { 
-				# if another item already exists at this location
-				# add a really small number to bump it up and make it unique
-				$startvalue += 0.001; 
-			}
-			$str_datahash{$chromovalue}{$startvalue} = $self->{data_table}->[$row];
-		}
+	# collect the data by chromosome: start and end
+	my %chrom2row;
+	for my $row (1 .. $self->last_row) {
+		my $c = $self->{data_table}->[$row]->[$chromo_i];
+		$chrom2row{$c} ||= [];
+		push @{ $chrom2row{$c} }, [
+			$self->{data_table}->[$row]->[$start_i], 
+			$self->{data_table}->[$row]->[$stop_i], 
+			$self->{data_table}->[$row]
+		];
 	}
 	
+	# get sane chromosome order
+	my @chroms = sane_chromo_sort(keys %chrom2row); 
 	
-	# Now re-load the data array with sorted data
-	# put the numeric chromosome data back first
-	my $i = 1; # keep track of the row
-	foreach my $chromovalue (sort {$a <=> $b} keys %num_datahash) {
-		# first, numeric sort on increasing chromosome number
-		foreach my $startvalue (
-			sort {$a <=> $b} keys %{ $num_datahash{$chromovalue} } 
-		) {
-			# second, numeric sort on increasing position value
-			$self->{data_table}->[$i] = $num_datahash{$chromovalue}{$startvalue};
-			$i++; # increment for next row
+	# sort the table
+	my @sorted_data;
+	push @sorted_data, $self->{data_table}->[0];
+	if ($self->gff) {
+		# sort by increasing start and decreasing end, i.e. decreasing length
+		# this should mostly handle putting genes/transcripts before exons
+		foreach my $chr (@chroms) {
+			push @sorted_data, 
+				map { $_->[2] }
+				sort { $a->[0] <=> $b->[0] or $b->[1] <=> $a->[1] }
+				@{ $chrom2row{$chr} };
 		}
 	}
-	# next put the string chromosome data back
-	foreach my $chromovalue (sort {$a cmp $b} keys %str_datahash) {
-		# first, ascii sort on increasing chromosome name
-		foreach my $startvalue (
-			sort {$a <=> $b} keys %{ $str_datahash{$chromovalue} } 
-		) {
-			# second, numeric sort on increasing position value
-			$self->{data_table}->[$i] = $str_datahash{$chromovalue}{$startvalue};
-			$i++; # increment for next row
+	else {
+		# sort by increasing start followed by increasing end, i.e. increasing length
+		foreach my $chr (@chroms) {
+			push @sorted_data, 
+				map { $_->[2] }
+				sort { $a->[0] <=> $b->[0] or $a->[1] <=> $b->[1] }
+				@{ $chrom2row{$chr} };
 		}
 	}
-	
+	$self->{data_table} = \@sorted_data;
 	return 1;
 }
 
