@@ -1,5 +1,5 @@
 package Bio::ToolBox::parser::bed;
-our $VERSION = '1.64';
+our $VERSION = '1.69';
 
 =head1 NAME
 
@@ -847,7 +847,7 @@ sub _parse_gappedPeak {
 	my @new = (
 		$data[3], # name
 		$data[0], # chrom
-		'+',      # strand is typically unstranded, so just pretend to be 
+		$data[5], # strand
 		$data[1], # txStart
 		$data[2], # txStop
 		$data[6], # cdsStart
@@ -864,7 +864,6 @@ sub _parse_gappedPeak {
 	# clean up feature and add extra values
 	$feature->add_tag_value('itemRGB', $data[8]);
 	$feature->score($data[4]);
-	$feature->strand($data[5]);
 	$feature->primary_tag('region'); # it is not a RNA
 	$feature->primary_id(sprintf("%s:%d-%d", $data[0], $data[1], $data[2]));
 		# change the primary ID to match other bed file behavior, not UCSC files'
